"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Context = void 0;
const ast_spec_1 = require("@typescript-eslint/types/dist/generated/ast-spec");
const publicRoleSplitter = "_";
const privateRoleSplitter = "__";
class Context {
    constructor(context, name, func) {
        this.roles = new Map();
        this.funcMap = new Map();
        this.funcNameMap = new Map();
        this.name = name;
        this.func = func;
        if (func.body.type != ast_spec_1.AST_NODE_TYPES.BlockStatement) {
            context.report({
                loc: func.body.loc,
                message: "A Context must be defined with a block statement (curly braces)",
            });
            return;
        }
        const potentialRoles = new Map(func.params.filter((p) => p.type == ast_spec_1.AST_NODE_TYPES.Identifier).map((i) => [i.name, { id: i, kind: "param" }]));
        const statements = func.body.body;
        // Check if a top-level statement exists as a potential Role.
        for (const s of statements) {
            const roleVar = Context.potentialRoleVar(s);
            if (roleVar) {
                potentialRoles.set(roleVar.id.name, {
                    id: roleVar.id,
                    kind: roleVar.kind,
                });
            }
        }
        //console.log(potentialRoles.keys());
        const functions = new Map();
        for (const s of statements) {
            if (s.type == ast_spec_1.AST_NODE_TYPES.FunctionDeclaration) {
                functions.set(s.id.name, { func: s, decl: null });
            }
            else if (s.type == ast_spec_1.AST_NODE_TYPES.VariableDeclaration &&
                s.declarations.length == 1 &&
                s.declarations[0]?.type == ast_spec_1.AST_NODE_TYPES.VariableDeclarator &&
                s.declarations[0]?.init?.type == ast_spec_1.AST_NODE_TYPES.ArrowFunctionExpression) {
                // Type definition is wrong, name exists on the Identifier.
                // eslint-disable-next-line @typescript-eslint/no-explicit-any
                const declarator = s.declarations[0];
                const name = declarator.id.name;
                if (!name) {
                    context.report({
                        loc: declarator.loc,
                        message: `Name not found for function`,
                    });
                }
                functions.set(name, { func: s.declarations[0].init, decl: s });
            }
        }
        // Check if a function is a RoleMethod
        for (const [name, expr] of functions) {
            const rm = this.roleMethodCall(name);
            if (!rm)
                continue;
            if (expr.decl && expr.decl.kind !== "const") {
                context.report({
                    loc: expr.decl.loc,
                    message: `A RoleMethod must be declared as const`,
                });
                continue;
            }
            const contextRm = {
                role: rm.role,
                method: rm.method,
                isPrivate: rm.isPrivate,
                func: expr.func,
            };
            this.funcMap.set(expr.func, contextRm);
            this.funcNameMap.set(name, contextRm);
            if (!this.roles.has(rm.role)) {
                // Check if a potential role exist, if so, add the Role.
                if (potentialRoles.has(rm.role)) {
                    // eslint-disable-next-line @typescript-eslint/no-non-null-assertion
                    const role = potentialRoles.get(rm.role);
                    this.roles.set(rm.role, {
                        name: rm.role,
                        id: role.id,
                        kind: role.kind,
                        methods: [contextRm],
                    });
                }
                else {
                    context.report({
                        loc: expr.func.loc,
                        message: `Role "${rm.role}" not found for RoleMethod "${rm.method}"`,
                    });
                }
            }
            else {
                this.roles.get(rm.role)?.methods?.push(contextRm);
            }
        }
    }
    static potentialRoleVar(node) {
        return node.type == ast_spec_1.AST_NODE_TYPES.VariableDeclaration &&
            node.declarations.length == 1 &&
            node.declarations[0]?.type == ast_spec_1.AST_NODE_TYPES.VariableDeclarator &&
            node.declarations[0]?.id.type == ast_spec_1.AST_NODE_TYPES.Identifier
            ? { id: node.declarations[0].id, kind: node.kind }
            : null;
    }
    roleMethodFromFunc(func) {
        return this.funcMap.get(func);
    }
    roleMethodFromName(name) {
        return this.funcNameMap.get(name);
    }
    isRoleMethod(name) {
        return name && name.includes(publicRoleSplitter);
    }
    roleMethodCall(name) {
        if (!name || !this.isRoleMethod(name))
            return null;
        const isPrivate = name.includes(privateRoleSplitter);
        const parts = name.split(isPrivate ? privateRoleSplitter : publicRoleSplitter);
        return { role: parts[0], method: parts[1], isPrivate };
    }
}
exports.Context = Context;
//# sourceMappingURL=context.js.map